package cn.newfeifan.mall.module.distri.service.integral;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.integral.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.module.distri.dal.mysql.integral.IntegralMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link IntegralServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(IntegralServiceImpl.class)
public class IntegralServiceImplTest extends BaseDbUnitTest {

    @Resource
    private IntegralServiceImpl integralService;

    @Resource
    private IntegralMapper integralMapper;

    @Test
    public void testCreateIntegral_success() {
        // 准备参数
        IntegralSaveReqVO createReqVO = randomPojo(IntegralSaveReqVO.class).setId(null);

        // 调用
        Long integralId = integralService.createIntegral(createReqVO);
        // 断言
        assertNotNull(integralId);
        // 校验记录的属性是否正确
        IntegralDO integral = integralMapper.selectById(integralId);
        assertPojoEquals(createReqVO, integral, "id");
    }

    @Test
    public void testUpdateIntegral_success() {
        // mock 数据
        IntegralDO dbIntegral = randomPojo(IntegralDO.class);
        integralMapper.insert(dbIntegral);// @Sql: 先插入出一条存在的数据
        // 准备参数
        IntegralSaveReqVO updateReqVO = randomPojo(IntegralSaveReqVO.class, o -> {
            o.setId(dbIntegral.getId()); // 设置更新的 ID
        });

        // 调用
        integralService.updateIntegral(updateReqVO);
        // 校验是否更新正确
        IntegralDO integral = integralMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, integral);
    }

    @Test
    public void testUpdateIntegral_notExists() {
        // 准备参数
        IntegralSaveReqVO updateReqVO = randomPojo(IntegralSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> integralService.updateIntegral(updateReqVO), INTEGRAL_NOT_EXISTS);
    }

    @Test
    public void testDeleteIntegral_success() {
        // mock 数据
        IntegralDO dbIntegral = randomPojo(IntegralDO.class);
        integralMapper.insert(dbIntegral);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbIntegral.getId();

        // 调用
        integralService.deleteIntegral(id);
       // 校验数据不存在了
       assertNull(integralMapper.selectById(id));
    }

    @Test
    public void testDeleteIntegral_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> integralService.deleteIntegral(id), INTEGRAL_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetIntegralPage() {
       // mock 数据
       IntegralDO dbIntegral = randomPojo(IntegralDO.class, o -> { // 等会查询到
           o.setUserId(null);
           o.setCurrentQuota(null);
           o.setFreezeQuota(null);
           o.setCategoryId(null);
           o.setCreateTime(null);
       });
       integralMapper.insert(dbIntegral);
       // 测试 userId 不匹配
       integralMapper.insert(cloneIgnoreId(dbIntegral, o -> o.setUserId(null)));
       // 测试 currentQuota 不匹配
       integralMapper.insert(cloneIgnoreId(dbIntegral, o -> o.setCurrentQuota(null)));
       // 测试 freezeQuota 不匹配
       integralMapper.insert(cloneIgnoreId(dbIntegral, o -> o.setFreezeQuota(null)));
       // 测试 categoryId 不匹配
       integralMapper.insert(cloneIgnoreId(dbIntegral, o -> o.setCategoryId(null)));
       // 测试 createTime 不匹配
       integralMapper.insert(cloneIgnoreId(dbIntegral, o -> o.setCreateTime(null)));
       // 准备参数
       IntegralPageReqVO reqVO = new IntegralPageReqVO();
       reqVO.setUserId(null);
       reqVO.setCurrentQuota(null);
       reqVO.setFreezeQuota(null);
       reqVO.setCategoryId(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<IntegralDO> pageResult = integralService.getIntegralPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbIntegral, pageResult.getList().get(0));
    }

}