package cn.newfeifan.mall.module.distri.service.ordercalc;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.ordercalc.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.ordercalc.OrderCalcDO;
import cn.newfeifan.mall.module.distri.dal.mysql.ordercalc.OrderCalcMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link OrderCalcServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(OrderCalcServiceImpl.class)
public class OrderCalcServiceImplTest extends BaseDbUnitTest {

    @Resource
    private OrderCalcServiceImpl orderCalcService;

    @Resource
    private OrderCalcMapper orderCalcMapper;

    @Test
    public void testCreateOrderCalc_success() {
        // 准备参数
        OrderCalcSaveReqVO createReqVO = randomPojo(OrderCalcSaveReqVO.class).setId(null);

        // 调用
        Long orderCalcId = orderCalcService.createOrderCalc(createReqVO);
        // 断言
        assertNotNull(orderCalcId);
        // 校验记录的属性是否正确
        OrderCalcDO orderCalc = orderCalcMapper.selectById(orderCalcId);
        assertPojoEquals(createReqVO, orderCalc, "id");
    }

    @Test
    public void testUpdateOrderCalc_success() {
        // mock 数据
        OrderCalcDO dbOrderCalc = randomPojo(OrderCalcDO.class);
        orderCalcMapper.insert(dbOrderCalc);// @Sql: 先插入出一条存在的数据
        // 准备参数
        OrderCalcSaveReqVO updateReqVO = randomPojo(OrderCalcSaveReqVO.class, o -> {
            o.setId(dbOrderCalc.getId()); // 设置更新的 ID
        });

        // 调用
        orderCalcService.updateOrderCalc(updateReqVO);
        // 校验是否更新正确
        OrderCalcDO orderCalc = orderCalcMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, orderCalc);
    }

    @Test
    public void testUpdateOrderCalc_notExists() {
        // 准备参数
        OrderCalcSaveReqVO updateReqVO = randomPojo(OrderCalcSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> orderCalcService.updateOrderCalc(updateReqVO), ORDER_CALC_NOT_EXISTS);
    }

    @Test
    public void testDeleteOrderCalc_success() {
        // mock 数据
        OrderCalcDO dbOrderCalc = randomPojo(OrderCalcDO.class);
        orderCalcMapper.insert(dbOrderCalc);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbOrderCalc.getId();

        // 调用
        orderCalcService.deleteOrderCalc(id);
       // 校验数据不存在了
       assertNull(orderCalcMapper.selectById(id));
    }

    @Test
    public void testDeleteOrderCalc_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> orderCalcService.deleteOrderCalc(id), ORDER_CALC_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetOrderCalcPage() {
       // mock 数据
       OrderCalcDO dbOrderCalc = randomPojo(OrderCalcDO.class, o -> { // 等会查询到
           o.setOrderNo(null);
           o.setUserId(null);
           o.setPrice(null);
           o.setCost(null);
           o.setGrossProfit(null);
           o.setGrossProfitUserQuota(null);
           o.setGrossProfitAncestorQuota(null);
           o.setGrossProfitBonusQuota(null);
           o.setGrossProfitPlatformQuota(null);
           o.setCreateTime(null);
       });
       orderCalcMapper.insert(dbOrderCalc);
       // 测试 orderNo 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setOrderNo(null)));
       // 测试 userId 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setUserId(null)));
       // 测试 price 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setPrice(null)));
       // 测试 cost 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setCost(null)));
       // 测试 grossProfit 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setGrossProfit(null)));
       // 测试 grossProfitUserQuota 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setGrossProfitUserQuota(null)));
       // 测试 grossProfitAncestorQuota 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setGrossProfitAncestorQuota(null)));
       // 测试 grossProfitBonusQuota 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setGrossProfitBonusQuota(null)));
       // 测试 grossProfitPlatformQuota 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setGrossProfitPlatformQuota(null)));
       // 测试 createTime 不匹配
       orderCalcMapper.insert(cloneIgnoreId(dbOrderCalc, o -> o.setCreateTime(null)));
       // 准备参数
       OrderCalcPageReqVO reqVO = new OrderCalcPageReqVO();
       reqVO.setOrderNo(null);
       reqVO.setUserId(null);
       reqVO.setPrice(null);
       reqVO.setCost(null);
       reqVO.setGrossProfit(null);
       reqVO.setGrossProfitUserQuota(null);
       reqVO.setGrossProfitAncestorQuota(null);
       reqVO.setGrossProfitBonusQuota(null);
       reqVO.setGrossProfitPlatformQuota(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<OrderCalcDO> pageResult = orderCalcService.getOrderCalcPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbOrderCalc, pageResult.getList().get(0));
    }

}