package cn.newfeifan.mall.module.distri.service.partitionbrothers;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.partitionbrothers.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.partitionbrothers.PartitionBrothersDO;
import cn.newfeifan.mall.module.distri.dal.mysql.partitionbrothers.PartitionBrothersMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link PartitionBrothersServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(PartitionBrothersServiceImpl.class)
public class PartitionBrothersServiceImplTest extends BaseDbUnitTest {

    @Resource
    private PartitionBrothersServiceImpl partitionBrothersService;

    @Resource
    private PartitionBrothersMapper partitionBrothersMapper;

    @Test
    public void testCreatePartitionBrothers_success() {
        // 准备参数
        PartitionBrothersSaveReqVO createReqVO = randomPojo(PartitionBrothersSaveReqVO.class).setId(null);

        // 调用
        Long partitionBrothersId = partitionBrothersService.createPartitionBrothers(createReqVO);
        // 断言
        assertNotNull(partitionBrothersId);
        // 校验记录的属性是否正确
        PartitionBrothersDO partitionBrothers = partitionBrothersMapper.selectById(partitionBrothersId);
        assertPojoEquals(createReqVO, partitionBrothers, "id");
    }

    @Test
    public void testUpdatePartitionBrothers_success() {
        // mock 数据
        PartitionBrothersDO dbPartitionBrothers = randomPojo(PartitionBrothersDO.class);
        partitionBrothersMapper.insert(dbPartitionBrothers);// @Sql: 先插入出一条存在的数据
        // 准备参数
        PartitionBrothersSaveReqVO updateReqVO = randomPojo(PartitionBrothersSaveReqVO.class, o -> {
            o.setId(dbPartitionBrothers.getId()); // 设置更新的 ID
        });

        // 调用
        partitionBrothersService.updatePartitionBrothers(updateReqVO);
        // 校验是否更新正确
        PartitionBrothersDO partitionBrothers = partitionBrothersMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, partitionBrothers);
    }

    @Test
    public void testUpdatePartitionBrothers_notExists() {
        // 准备参数
        PartitionBrothersSaveReqVO updateReqVO = randomPojo(PartitionBrothersSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> partitionBrothersService.updatePartitionBrothers(updateReqVO), PARTITION_BROTHERS_NOT_EXISTS);
    }

    @Test
    public void testDeletePartitionBrothers_success() {
        // mock 数据
        PartitionBrothersDO dbPartitionBrothers = randomPojo(PartitionBrothersDO.class);
        partitionBrothersMapper.insert(dbPartitionBrothers);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbPartitionBrothers.getId();

        // 调用
        partitionBrothersService.deletePartitionBrothers(id);
       // 校验数据不存在了
       assertNull(partitionBrothersMapper.selectById(id));
    }

    @Test
    public void testDeletePartitionBrothers_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> partitionBrothersService.deletePartitionBrothers(id), PARTITION_BROTHERS_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetPartitionBrothersPage() {
       // mock 数据
       PartitionBrothersDO dbPartitionBrothers = randomPojo(PartitionBrothersDO.class, o -> { // 等会查询到
           o.setUserId(null);
           o.setPartNo(null);
           o.setPrice(null);
           o.setHasHit(null);
           o.setAfterHitQuota(null);
           o.setSize(null);
           o.setCreateTime(null);
       });
       partitionBrothersMapper.insert(dbPartitionBrothers);
       // 测试 userId 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setUserId(null)));
       // 测试 partNo 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setPartNo(null)));
       // 测试 price 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setPrice(null)));
       // 测试 hasHit 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setHasHit(null)));
       // 测试 afterHitQuota 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setAfterHitQuota(null)));
       // 测试 size 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setSize(null)));
       // 测试 createTime 不匹配
       partitionBrothersMapper.insert(cloneIgnoreId(dbPartitionBrothers, o -> o.setCreateTime(null)));
       // 准备参数
       PartitionBrothersPageReqVO reqVO = new PartitionBrothersPageReqVO();
       reqVO.setUserId(null);
       reqVO.setPartNo(null);
       reqVO.setPrice(null);
       reqVO.setHasHit(null);
       reqVO.setAfterHitQuota(null);
       reqVO.setSize(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<PartitionBrothersDO> pageResult = partitionBrothersService.getPartitionBrothersPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbPartitionBrothers, pageResult.getList().get(0));
    }

}