package cn.newfeifan.mall.module.distri.service.partitionson;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.partitionson.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.partitionson.PartitionSonDO;
import cn.newfeifan.mall.module.distri.dal.mysql.partitionson.PartitionSonMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link PartitionSonServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(PartitionSonServiceImpl.class)
public class PartitionSonServiceImplTest extends BaseDbUnitTest {

    @Resource
    private PartitionSonServiceImpl partitionSonService;

    @Resource
    private PartitionSonMapper partitionSonMapper;

    @Test
    public void testCreatePartitionSon_success() {
        // 准备参数
        PartitionSonSaveReqVO createReqVO = randomPojo(PartitionSonSaveReqVO.class).setId(null);

        // 调用
        Long partitionSonId = partitionSonService.createPartitionSon(createReqVO);
        // 断言
        assertNotNull(partitionSonId);
        // 校验记录的属性是否正确
        PartitionSonDO partitionSon = partitionSonMapper.selectById(partitionSonId);
        assertPojoEquals(createReqVO, partitionSon, "id");
    }

    @Test
    public void testUpdatePartitionSon_success() {
        // mock 数据
        PartitionSonDO dbPartitionSon = randomPojo(PartitionSonDO.class);
        partitionSonMapper.insert(dbPartitionSon);// @Sql: 先插入出一条存在的数据
        // 准备参数
        PartitionSonSaveReqVO updateReqVO = randomPojo(PartitionSonSaveReqVO.class, o -> {
            o.setId(dbPartitionSon.getId()); // 设置更新的 ID
        });

        // 调用
        partitionSonService.updatePartitionSon(updateReqVO);
        // 校验是否更新正确
        PartitionSonDO partitionSon = partitionSonMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, partitionSon);
    }

    @Test
    public void testUpdatePartitionSon_notExists() {
        // 准备参数
        PartitionSonSaveReqVO updateReqVO = randomPojo(PartitionSonSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> partitionSonService.updatePartitionSon(updateReqVO), PARTITION_SON_NOT_EXISTS);
    }

    @Test
    public void testDeletePartitionSon_success() {
        // mock 数据
        PartitionSonDO dbPartitionSon = randomPojo(PartitionSonDO.class);
        partitionSonMapper.insert(dbPartitionSon);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbPartitionSon.getId();

        // 调用
        partitionSonService.deletePartitionSon(id);
       // 校验数据不存在了
       assertNull(partitionSonMapper.selectById(id));
    }

    @Test
    public void testDeletePartitionSon_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> partitionSonService.deletePartitionSon(id), PARTITION_SON_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetPartitionSonPage() {
       // mock 数据
       PartitionSonDO dbPartitionSon = randomPojo(PartitionSonDO.class, o -> { // 等会查询到
           o.setUserId(null);
           o.setPartNo(null);
           o.setPrice(null);
           o.setHasHit(null);
           o.setAfterHitQuota(null);
           o.setSize(null);
           o.setCreateTime(null);
       });
       partitionSonMapper.insert(dbPartitionSon);
       // 测试 userId 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setUserId(null)));
       // 测试 partNo 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setPartNo(null)));
       // 测试 price 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setPrice(null)));
       // 测试 hasHit 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setHasHit(null)));
       // 测试 afterHitQuota 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setAfterHitQuota(null)));
       // 测试 size 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setSize(null)));
       // 测试 createTime 不匹配
       partitionSonMapper.insert(cloneIgnoreId(dbPartitionSon, o -> o.setCreateTime(null)));
       // 准备参数
       PartitionSonPageReqVO reqVO = new PartitionSonPageReqVO();
       reqVO.setUserId(null);
       reqVO.setPartNo(null);
       reqVO.setPrice(null);
       reqVO.setHasHit(null);
       reqVO.setAfterHitQuota(null);
       reqVO.setSize(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<PartitionSonDO> pageResult = partitionSonService.getPartitionSonPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbPartitionSon, pageResult.getList().get(0));
    }

}