package cn.newfeifan.mall.module.distri.service.sharepath;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.sharepath.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.sharepath.SharePathDO;
import cn.newfeifan.mall.module.distri.dal.mysql.sharepath.SharePathMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link SharePathServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(SharePathServiceImpl.class)
public class SharePathServiceImplTest extends BaseDbUnitTest {

    @Resource
    private SharePathServiceImpl sharePathService;

    @Resource
    private SharePathMapper sharePathMapper;

    @Test
    public void testCreateSharePath_success() {
        // 准备参数
        SharePathSaveReqVO createReqVO = randomPojo(SharePathSaveReqVO.class).setId(null);

        // 调用
        Long sharePathId = sharePathService.createSharePath(createReqVO);
        // 断言
        assertNotNull(sharePathId);
        // 校验记录的属性是否正确
        SharePathDO sharePath = sharePathMapper.selectById(sharePathId);
        assertPojoEquals(createReqVO, sharePath, "id");
    }

    @Test
    public void testUpdateSharePath_success() {
        // mock 数据
        SharePathDO dbSharePath = randomPojo(SharePathDO.class);
        sharePathMapper.insert(dbSharePath);// @Sql: 先插入出一条存在的数据
        // 准备参数
        SharePathSaveReqVO updateReqVO = randomPojo(SharePathSaveReqVO.class, o -> {
            o.setId(dbSharePath.getId()); // 设置更新的 ID
        });

        // 调用
        sharePathService.updateSharePath(updateReqVO);
        // 校验是否更新正确
        SharePathDO sharePath = sharePathMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, sharePath);
    }

    @Test
    public void testUpdateSharePath_notExists() {
        // 准备参数
        SharePathSaveReqVO updateReqVO = randomPojo(SharePathSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> sharePathService.updateSharePath(updateReqVO), SHARE_PATH_NOT_EXISTS);
    }

    @Test
    public void testDeleteSharePath_success() {
        // mock 数据
        SharePathDO dbSharePath = randomPojo(SharePathDO.class);
        sharePathMapper.insert(dbSharePath);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbSharePath.getId();

        // 调用
        sharePathService.deleteSharePath(id);
       // 校验数据不存在了
       assertNull(sharePathMapper.selectById(id));
    }

    @Test
    public void testDeleteSharePath_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> sharePathService.deleteSharePath(id), SHARE_PATH_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetSharePathPage() {
       // mock 数据
       SharePathDO dbSharePath = randomPojo(SharePathDO.class, o -> { // 等会查询到
           o.setDepth(null);
           o.setCreateTime(null);
           o.setAncestor(null);
           o.setDescendant(null);
           o.setSort(null);
       });
       sharePathMapper.insert(dbSharePath);
       // 测试 depth 不匹配
       sharePathMapper.insert(cloneIgnoreId(dbSharePath, o -> o.setDepth(null)));
       // 测试 createTime 不匹配
       sharePathMapper.insert(cloneIgnoreId(dbSharePath, o -> o.setCreateTime(null)));
       // 测试 ancestor 不匹配
       sharePathMapper.insert(cloneIgnoreId(dbSharePath, o -> o.setAncestor(null)));
       // 测试 descendant 不匹配
       sharePathMapper.insert(cloneIgnoreId(dbSharePath, o -> o.setDescendant(null)));
       // 测试 sort 不匹配
       sharePathMapper.insert(cloneIgnoreId(dbSharePath, o -> o.setSort(null)));
       // 准备参数
       SharePathPageReqVO reqVO = new SharePathPageReqVO();
       reqVO.setDepth(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setAncestor(null);
       reqVO.setDescendant(null);
       reqVO.setSort(null);

       // 调用
       PageResult<SharePathDO> pageResult = sharePathService.getSharePathPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbSharePath, pageResult.getList().get(0));
    }

}