package cn.newfeifan.mall.module.distri.service.socialstatus;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.socialstatus.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.socialstatus.SocialStatusDO;
import cn.newfeifan.mall.module.distri.dal.mysql.socialstatus.SocialStatusMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link SocialStatusServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(SocialStatusServiceImpl.class)
public class SocialStatusServiceImplTest extends BaseDbUnitTest {

    @Resource
    private SocialStatusServiceImpl socialStatusService;

    @Resource
    private SocialStatusMapper socialStatusMapper;

    @Test
    public void testCreateSocialStatus_success() {
        // 准备参数
        SocialStatusSaveReqVO createReqVO = randomPojo(SocialStatusSaveReqVO.class).setId(null);

        // 调用
        Long socialStatusId = socialStatusService.createSocialStatus(createReqVO);
        // 断言
        assertNotNull(socialStatusId);
        // 校验记录的属性是否正确
        SocialStatusDO socialStatus = socialStatusMapper.selectById(socialStatusId);
        assertPojoEquals(createReqVO, socialStatus, "id");
    }

    @Test
    public void testUpdateSocialStatus_success() {
        // mock 数据
        SocialStatusDO dbSocialStatus = randomPojo(SocialStatusDO.class);
        socialStatusMapper.insert(dbSocialStatus);// @Sql: 先插入出一条存在的数据
        // 准备参数
        SocialStatusSaveReqVO updateReqVO = randomPojo(SocialStatusSaveReqVO.class, o -> {
            o.setId(dbSocialStatus.getId()); // 设置更新的 ID
        });

        // 调用
        socialStatusService.updateSocialStatus(updateReqVO);
        // 校验是否更新正确
        SocialStatusDO socialStatus = socialStatusMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, socialStatus);
    }

    @Test
    public void testUpdateSocialStatus_notExists() {
        // 准备参数
        SocialStatusSaveReqVO updateReqVO = randomPojo(SocialStatusSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> socialStatusService.updateSocialStatus(updateReqVO), SOCIAL_STATUS_NOT_EXISTS);
    }

    @Test
    public void testDeleteSocialStatus_success() {
        // mock 数据
        SocialStatusDO dbSocialStatus = randomPojo(SocialStatusDO.class);
        socialStatusMapper.insert(dbSocialStatus);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbSocialStatus.getId();

        // 调用
        socialStatusService.deleteSocialStatus(id);
       // 校验数据不存在了
       assertNull(socialStatusMapper.selectById(id));
    }

    @Test
    public void testDeleteSocialStatus_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> socialStatusService.deleteSocialStatus(id), SOCIAL_STATUS_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetSocialStatusPage() {
       // mock 数据
       SocialStatusDO dbSocialStatus = randomPojo(SocialStatusDO.class, o -> { // 等会查询到
           o.setLevel(null);
           o.setName(null);
           o.setCreateTime(null);
           o.setPoint(null);
       });
       socialStatusMapper.insert(dbSocialStatus);
       // 测试 level 不匹配
       socialStatusMapper.insert(cloneIgnoreId(dbSocialStatus, o -> o.setLevel(null)));
       // 测试 name 不匹配
       socialStatusMapper.insert(cloneIgnoreId(dbSocialStatus, o -> o.setName(null)));
       // 测试 createTime 不匹配
       socialStatusMapper.insert(cloneIgnoreId(dbSocialStatus, o -> o.setCreateTime(null)));
       // 测试 point 不匹配
       socialStatusMapper.insert(cloneIgnoreId(dbSocialStatus, o -> o.setPoint(null)));
       // 准备参数
       SocialStatusPageReqVO reqVO = new SocialStatusPageReqVO();
       reqVO.setLevel(null);
       reqVO.setName(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setPoint(null);

       // 调用
       PageResult<SocialStatusDO> pageResult = socialStatusService.getSocialStatusPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbSocialStatus, pageResult.getList().get(0));
    }

}