package cn.newfeifan.mall.module.distri.service.user;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.user.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.user.UserDO;
import cn.newfeifan.mall.module.distri.dal.mysql.user.UserMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link UserServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(UserServiceImpl.class)
public class UserServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UserServiceImpl userService;

    @Resource
    private UserMapper userMapper;

    @Test
    public void testCreateUser_success() {
        // 准备参数
        UserSaveReqVO createReqVO = randomPojo(UserSaveReqVO.class).setId(null);

        // 调用
        Long userId = userService.createUser(createReqVO);
        // 断言
        assertNotNull(userId);
        // 校验记录的属性是否正确
        UserDO user = userMapper.selectById(userId);
        assertPojoEquals(createReqVO, user, "id");
    }

    @Test
    public void testUpdateUser_success() {
        // mock 数据
        UserDO dbUser = randomPojo(UserDO.class);
        userMapper.insert(dbUser);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UserSaveReqVO updateReqVO = randomPojo(UserSaveReqVO.class, o -> {
            o.setId(dbUser.getId()); // 设置更新的 ID
        });

        // 调用
        userService.updateUser(updateReqVO);
        // 校验是否更新正确
        UserDO user = userMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, user);
    }

    @Test
    public void testUpdateUser_notExists() {
        // 准备参数
        UserSaveReqVO updateReqVO = randomPojo(UserSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> userService.updateUser(updateReqVO), USER_NOT_EXISTS);
    }

    @Test
    public void testDeleteUser_success() {
        // mock 数据
        UserDO dbUser = randomPojo(UserDO.class);
        userMapper.insert(dbUser);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUser.getId();

        // 调用
        userService.deleteUser(id);
       // 校验数据不存在了
       assertNull(userMapper.selectById(id));
    }

    @Test
    public void testDeleteUser_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> userService.deleteUser(id), USER_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserPage() {
       // mock 数据
       UserDO dbUser = randomPojo(UserDO.class, o -> { // 等会查询到
           o.setCreateTime(null);
           o.setUserId(null);
           o.setName(null);
           o.setMobile(null);
           o.setSource(null);
           o.setIntegralId(null);
           o.setSocialStatus(null);
           o.setSocialStatusPoint(null);
           o.setCategoryId(null);
       });
       userMapper.insert(dbUser);
       // 测试 createTime 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setCreateTime(null)));
       // 测试 userId 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setUserId(null)));
       // 测试 name 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setName(null)));
       // 测试 mobile 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setMobile(null)));
       // 测试 source 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setSource(null)));
       // 测试 integralId 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setIntegralId(null)));
       // 测试 socialStatus 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setSocialStatus(null)));
       // 测试 socialStatusPoint 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setSocialStatusPoint(null)));
       // 测试 categoryId 不匹配
       userMapper.insert(cloneIgnoreId(dbUser, o -> o.setCategoryId(null)));
       // 准备参数
       UserPageReqVO reqVO = new UserPageReqVO();
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setUserId(null);
       reqVO.setName(null);
       reqVO.setMobile(null);
       reqVO.setSource(null);
       reqVO.setIntegralId(null);
       reqVO.setSocialStatus(null);
       reqVO.setSocialStatusPoint(null);
       reqVO.setCategoryId(null);

       // 调用
       PageResult<UserDO> pageResult = userService.getUserPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbUser, pageResult.getList().get(0));
    }

}